﻿
Public Class Form1

    Private Sub Form1_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
        Me.Text = AppTitleAndVersion()
        EventsAreEnabled = False
        InitPanels()
        Load_INI()
        UpdateAllPanels()
        EnableDisablePanels()
        EnablePaperWidthAndHeight()
        SetParams()
        SetVectorizationParams()
        EnsureFileInAppFolders()
        Image_Open()
        UpdateVectorsAndImage()
        EventsAreEnabled = True
        Refresh()
        Opacity = 1
    End Sub

    Private Sub Form1_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing
        EventsAreEnabled = False
        Save_INI()
    End Sub

    Private Sub Form1_LocationChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.LocationChanged
        If Not EventsAreEnabled Then Exit Sub
        LimitFormPosition(Me)
    End Sub

    Private Sub Form1_Resize(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Resize
        If Not EventsAreEnabled Then Exit Sub
        UpdateVectorsAndImage()
    End Sub

    Private Sub SetDefaultFocus()
        lbl_Resolution.Focus()
    End Sub

    ' ========================================================================================================
    '   LOAD and SAVE
    ' ========================================================================================================
    Private Sub btn_Load_ClickButtonArea(ByVal Sender As System.Object, ByVal e As System.EventArgs) Handles btn_Load.ClickButtonArea
        Dim ofd As OpenFileDialog = New OpenFileDialog()
        ofd.InitialDirectory = Application.StartupPath & "\SavedProperties"
        ofd.Filter = "Properties file (*.txt)|*txt"
        ofd.Title = "Load program file"
        ofd.DefaultExt = ".txt"
        ofd.Multiselect = True
        If Not IO.Directory.Exists(ofd.InitialDirectory) Then Return
        If ofd.ShowDialog() = Windows.Forms.DialogResult.OK Then
            Load_INI(ofd.FileName)
            SetParams()
            SetVectorizationParams()
            EnsureFileInAppFolders()
            Image_Open()
            UpdateVectorsAndImage()
        End If
    End Sub
    Private Sub btn_Save_ClickButtonArea(ByVal Sender As System.Object, ByVal e As System.EventArgs) Handles btn_Save.ClickButtonArea
        Dim OldColor As Color = btn_Save.ForeColor
        btn_Save.ForeColor = Color.DarkRed
        btn_Save.Refresh()
        My.Computer.FileSystem.CreateDirectory(Application.StartupPath & "\SavedProperties")
        Dim sfd As SaveFileDialog = New SaveFileDialog()
        sfd.InitialDirectory = Application.StartupPath & "\SavedProperties"
        sfd.FileName = IO.Path.GetFileNameWithoutExtension(ImageFile) + "__" + lbl_Time.Text.Replace(" ", "_") + ".txt"
        'SFD.RestoreDirectory = False
        sfd.AutoUpgradeEnabled = True
        sfd.Title = "Save program file"
        sfd.ValidateNames = False
        sfd.DefaultExt = ".txt"
        sfd.AddExtension = True
        sfd.Filter = "All files (*.*)|*.*"
        If sfd.ShowDialog() = Windows.Forms.DialogResult.OK Then
            Save_INI(sfd.FileName)
        End If
        Threading.Thread.Sleep(300)
        btn_Save.ForeColor = OldColor
    End Sub

    ' ========================================================================================================
    '   LOAD IMAGE
    ' ========================================================================================================
    Private Sub btn_LoadImage_ClickButtonArea(ByVal Sender As System.Object, ByVal e As System.EventArgs) Handles btn_LoadImage.ClickButtonArea
        EnsureFileInAppFolders()
        OpenImageDialog()
        Save_INI()
        Image_Open()
        UpdateVectorsAndImage()
        SetParams()
        SetDefaultFocus()
    End Sub

    Private Sub OpenImageDialog()
        ImageFile = PlatformAdjustedFileName(ImageFile, Application.StartupPath & "/../media/")
        Dim ofd As OpenFileDialog = New OpenFileDialog
        ofd.Filter = "Image files (*.jpeg;*.gif;*.png;*.bmp;*.tiff;*.emf;*.wmf)|*.jpeg;*.jpg;*.gif;*.png;*.bmp;*.tiff;*.emf;*.wmf;*.tif|All files (*.*)|*.*"
        If IO.File.Exists(ImageFile) Then
            ofd.InitialDirectory = IO.Path.GetDirectoryName(ImageFile)
            ofd.FileName = IO.Path.GetFileName(ImageFile)
        Else
            ofd.InitialDirectory = PlatformAdjustedFileName(Application.StartupPath & "\Media")
            ofd.FileName = ""
        End If
        ofd.Title = "Load image"
        ofd.Multiselect = True
        '
        SendKeys.Send("{HOME}{END}")   ' name visible and unselected
        '
        If ofd.ShowDialog() = Windows.Forms.DialogResult.OK Then
            ImageFile = ofd.FileName
        End If
    End Sub


    ' ========================================================================================================
    '   SAVE PARAMS ON LOST-FOCUS
    ' ========================================================================================================
    Private Sub Params_LostFocus(ByVal sender As System.Object, ByVal e As System.EventArgs) _
                                                                    Handles txt_RowsNumber.LostFocus, _
                                                                            txt_WavesSize.LostFocus, _
                                                                            txt_WavesCount.LostFocus, _
                                                                            txt_WhiteLevel.LostFocus, _
                                                                            txt_MidLevel.LostFocus, _
                                                                            txt_BlackLevel.LostFocus, _
                                                                            chk_HorizontalRows.LostFocus, _
                                                                            chk_VerticalRows.LostFocus, _
                                                                            txt_Width.LostFocus, _
                                                                            txt_Height.LostFocus, _
                                                                            chk_UseImageAspect.LostFocus, _
                                                                            chk_UseImageSize.LostFocus, _
                                                                            txt_PaperWidth.LostFocus, _
                                                                            txt_PaperHeight.LostFocus, _
                                                                            chk_CenterOnPaper.LostFocus, _
                                                                            txt_Up.LostFocus, _
                                                                            txt_Down.LostFocus, _
                                                                            txt_ToolSize.LostFocus, _
                                                                            txt_Feed.LostFocus, _
                                                                            txt_Rapids.LostFocus, _
                                                                            txt_Acceleration.LostFocus
        If Not EventsAreEnabled Then Return
        Save_INI()
    End Sub


    ' ========================================================================================================
    '   VECTORIZATION COMMANDS
    ' ========================================================================================================
    Private Sub btn_ViewVectorized_CheckedChanged(ByVal Sender As System.Object, ByVal e As System.EventArgs) Handles btn_ViewVectorized.CheckedChanged
        UpdateVectorsAndImage()
        EnableDisablePanels()
    End Sub

    Private Sub EnableDisablePanels()
        If btn_ViewVectorized.Checked Then
            GroupBox2.Enabled = True
            GroupBox3.Enabled = True
            GroupBox4.Enabled = True
            GroupBox5.Enabled = True
        Else
            GroupBox2.Enabled = False
            GroupBox3.Enabled = False
            GroupBox4.Enabled = False
            GroupBox5.Enabled = False
        End If
    End Sub

    Private Sub VectorizationParams_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) _
                                                                                 Handles txt_RowsNumber.TextChanged, _
                                                                                         txt_WavesCount.TextChanged, _
                                                                                         txt_WavesSize.TextChanged, _
                                                                                         txt_WhiteLevel.TextChanged, _
                                                                                         txt_MidLevel.TextChanged, _
                                                                                         txt_BlackLevel.TextChanged
        If Not EventsAreEnabled Then Return
        CType(sender, TextBox).Refresh()
        SetVectorizationParams()
        UpdateVectorsAndImage()
        RestartUpdateTimer()
    End Sub

    Private Sub SetVectorizationParams()
        RowsNumber = txt_RowsNumber.NumericValueInteger
        PointsNumber = txt_WavesCount.NumericValueInteger
        ZigZagPercentual = txt_WavesSize.NumericValueInteger
        WhiteLevel = txt_WhiteLevel.NumericValueInteger
        MidLevel = txt_MidLevel.NumericValueInteger
        BlackLevel = txt_BlackLevel.NumericValueInteger
        '
        If WhiteLevel < MidLevel Then
            WhiteLevel = MidLevel '+ 1
            txt_WhiteLevel.NumericValueInteger = WhiteLevel
        End If
        If BlackLevel >= MidLevel Then
            BlackLevel = MidLevel - 1
            txt_BlackLevel.NumericValueInteger = BlackLevel
        End If
        If MidLevel > WhiteLevel Then
            MidLevel = WhiteLevel '- 1
            txt_MidLevel.NumericValueInteger = MidLevel
        End If
        If MidLevel <= BlackLevel Then
            MidLevel = BlackLevel + 1
            txt_MidLevel.NumericValueInteger = MidLevel
        End If
    End Sub

    Private Sub chk_HorizontalRows_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles chk_HorizontalRows.CheckedChanged, _
                                                                                                                      chk_VerticalRows.CheckedChanged
        If Not EventsAreEnabled Then Return
        UpdateVectorsAndImage()
        Save_INI()
    End Sub

    Private Sub UpdateVectorsAndImage()
        If btn_ViewVectorized.Checked Then
            ' -------------------------------------------------------------
            'Dim sw As Stopwatch = New Stopwatch
            'sw.Start()
            If GrayBitmap Is Nothing Then Return
            ' -------------------------------------------------------------
            VecList = New VectorList
            If chk_HorizontalRows.Checked Then CreateVectors_H()
            If chk_VerticalRows.Checked Then CreateVectors_V()
            ' -------------------------------------------------------------
            Dim aspect As Double = GrayBitmap.Width / GrayBitmap.Height
            Dim w, h As Int32
            w = Pbox1.ClientRectangle.Width
            h = Pbox1.ClientRectangle.Height
            If h * aspect > w Then
                h = CInt(w / aspect)
            Else
                w = CInt(h * aspect)
            End If
            Pbox1.BackgroundImage = VectorsToImage(w, h)
            RestartUpdateTimer()
            ' -------------------------------------------------------------
            'Text = sw.Elapsed.TotalMilliseconds.ToString
            ' -------------------------------------------------------------
        Else
            Pbox1.BackgroundImage = GrayBitmap
        End If
        Pbox1.Refresh()
    End Sub


    ' ========================================================================================================
    '   GCODE commands
    ' ========================================================================================================
    Private Sub txt_Width_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) _
                                                                       Handles txt_Width.TextChanged
        If Not EventsAreEnabled Then Return
        EventsAreEnabled = False
        txt_Width.Refresh()
        Gcode_SetWidth(txt_Width.NumericValueInteger)
        If chk_UseImageAspect.Checked Then
            txt_Height.NumericValue = Gcode_GetOutHeight()
            txt_Height.Refresh()
        End If
        EventsAreEnabled = True
        '
        RestartUpdateTimer()
    End Sub
    Private Sub txt_Height_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) _
                                                                       Handles txt_Height.TextChanged
        If Not EventsAreEnabled Then Return
        EventsAreEnabled = False
        txt_Height.Refresh()
        Gcode_SetHeight(txt_Height.NumericValueInteger)
        If chk_UseImageAspect.Checked Then
            txt_Width.NumericValue = Gcode_GetOutWidth()
            txt_Width.Refresh()
        End If
        EventsAreEnabled = True
        '
        RestartUpdateTimer()
    End Sub

    Private Sub chk_CenterOnPaper_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles chk_CenterOnPaper.CheckedChanged
        If Not EventsAreEnabled Then Return
        EnablePaperWidthAndHeight()
    End Sub
    Private Sub EnablePaperWidthAndHeight()
        txt_PaperWidth.Enabled = chk_CenterOnPaper.Checked
        txt_PaperHeight.Enabled = chk_CenterOnPaper.Checked
    End Sub

    Private Sub Params_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) _
                                                                         Handles chk_UseImageAspect.CheckedChanged, _
                                                                                 chk_UseImageSize.CheckedChanged, _
                                                                                 txt_Up.TextChanged, _
                                                                                 txt_Down.TextChanged, _
                                                                                 txt_ToolSize.TextChanged, _
                                                                                 txt_Feed.TextChanged, _
                                                                                 txt_Rapids.TextChanged, _
                                                                                 txt_Acceleration.TextChanged
        If Not EventsAreEnabled Then Return
        SetParams()
    End Sub

    Private Sub SetParams()
        Gcode_UseImageAspect = chk_UseImageAspect.Checked
        Gcode_UseImageSize = chk_UseImageSize.Checked
        If chk_UseImageSize.Checked Then
            chk_UseImageAspect.Checked = True
            txt_Width.NumericValue = Gcode_GetOutWidth()
            txt_Height.NumericValue = Gcode_GetOutHeight()
            chk_UseImageAspect.Enabled = False
            txt_Width.Enabled = False
            txt_Height.Enabled = False
        Else
            Gcode_SetWidth(txt_Width.NumericValueInteger)
            Gcode_SetHeight(txt_Height.NumericValueInteger)
            chk_UseImageAspect.Enabled = True
            txt_Width.Enabled = True
            txt_Height.Enabled = True
            If chk_UseImageAspect.Checked Then
                Gcode_SetWidth(txt_Width.NumericValueInteger)
                txt_Height.NumericValue = Gcode_GetOutHeight()
                txt_Height.Refresh()
            End If
        End If
        Gcode_Zup = CSng(txt_Up.NumericValue)
        Gcode_Zdown = CSng(txt_Down.NumericValue)
        Gcode_ToolSize = CSng(txt_ToolSize.NumericValue)
        Gcode_Feed = txt_Feed.NumericValueInteger
        Gcode_Rapids = txt_Rapids.NumericValueInteger
        Gcode_Acceleration = txt_Acceleration.NumericValueInteger
        '
        RestartUpdateTimer()
    End Sub

    Private Sub btn_CreateGcode_ClickButtonArea(ByVal Sender As System.Object, ByVal e As System.EventArgs) Handles btn_CreateGcode.ClickButtonArea
        Gcode_CenterImage(chk_CenterOnPaper.Checked, txt_PaperWidth.NumericValueInteger, txt_PaperHeight.NumericValueInteger)
        Gcode_FromVectorList(ImageFile)
        Gcode_WriteFile()
        SetDefaultFocus()
    End Sub

    ' ========================================================================================================
    '   Update timer
    ' ========================================================================================================
    Private WithEvents UpdateTimer As Timer = New Timer
    Private Sub RestartUpdateTimer()
        UpdateTimer.Interval = 100
        UpdateTimer.Stop()
        UpdateTimer.Start()
    End Sub
    Private Sub UpdateTimer_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles UpdateTimer.Tick
        UpdateTimer.Stop()
        Gcode_CenterImage(chk_CenterOnPaper.Checked, txt_PaperWidth.NumericValueInteger, txt_PaperHeight.NumericValueInteger)
        Gcode_FromVectorList(ImageFile)
        lbl_Time.Text = Gcode_GetTotalTime()
    End Sub


    ' ===================================================================================================
    '  PANELS 
    ' ===================================================================================================
    Private PanelHeight(5) As Int32

    Private Sub InitPanels()
        PanelHeight(1) = GroupBox1.Height
        PanelHeight(2) = GroupBox2.Height
        PanelHeight(3) = GroupBox3.Height
        PanelHeight(4) = GroupBox4.Height
        PanelHeight(5) = GroupBox5.Height
    End Sub

    Private Sub GroupBox_Click(ByVal sender As Object, ByVal e As EventArgs) Handles GroupBox1.Click, _
                                                                                     GroupBox2.Click, _
                                                                                     GroupBox3.Click, _
                                                                                     GroupBox4.Click, _
                                                                                     GroupBox5.Click
        Dim gbox As GroupBox = CType(sender, GroupBox)
        If gbox.PointToClient(MousePosition).Y > 20 Then Return
        ToggleGboxHeight(gbox)
        UpdateAllPanels()
    End Sub

    Private Sub ToggleGboxHeight(ByVal gbox As GroupBox)
        Dim i As Int32 = CInt(Val(gbox.Name.Substring(gbox.Name.Length - 1)))
        If gbox.Height > 20 Then
            gbox.Height = 16
        Else
            gbox.Height = PanelHeight(i)
        End If
    End Sub

    Private Sub SetGboxHeight(ByVal gbox As GroupBox)
        Dim i As Int32 = CInt(Val(gbox.Name.Substring(gbox.Name.Length - 1)))
        If gbox.Height <= 20 Then
            gbox.Height = 16
        Else
            gbox.Height = PanelHeight(i)
        End If
    End Sub

    Private Sub UpdateAllPanels()
        ' --------------------------------------- Use this method if some GBOX is not located on the form
        'GroupBox4.Parent = Me
        'GroupBox4.Left = GroupBox1.Left
        'GroupBox5.Parent = Me
        'GroupBox5.Left = GroupBox1.Left
        ' --------------------------------------- Ensure the correct height also if INI is not ok
        SetGboxHeight(GroupBox1)
        SetGboxHeight(GroupBox2)
        SetGboxHeight(GroupBox3)
        SetGboxHeight(GroupBox4)
        SetGboxHeight(GroupBox5)
        ' --------------------------------------- Set the vertical positions
        GroupBox2.Top = GroupBox1.Bottom + 6
        GroupBox3.Top = GroupBox2.Bottom + 6
        GroupBox4.Top = GroupBox3.Bottom + 6
        GroupBox5.Top = GroupBox4.Bottom + 6
        ' --------------------------------------- Set the Form minimum height
        Dim topSize As Int32 = Me.Size.Height - Me.ClientSize.Height
        Dim MinHeight As Int32 = GroupBox5.Bottom + topSize + 5
        If MinHeight < 480 Then MinHeight = 480
        Me.MinimumSize = New Size(Me.MinimumSize.Width, MinHeight)
    End Sub


End Class