﻿Imports System.Collections.Generic
Imports System.Math

Module Module_Gcode

    Private Gcode_Width As Single
    Private Gcode_Height As Single
    Private Gcode_ZeroX As Single
    Private Gcode_ZeroY As Single
    Friend Gcode_UseImageAspect As Boolean = False
    Friend Gcode_UseImageSize As Boolean = False
    Friend Gcode_Zup As Single = 1
    Friend Gcode_Zdown As Single = -0.5
    Friend Gcode_Feed As Single = 2000
    Friend Gcode_Rapids As Single = 2000
    Friend Gcode_Acceleration As Single = 1000
    Friend Gcode_Speed As Single = 20000
    Friend Gcode_ToolSize As Single = 0.8

    Const krad As Double = Math.PI / 180

    Friend Sub Gcode_SetWidth(ByVal w As Single)
        Gcode_Width = w
        If Gcode_UseImageAspect AndAlso GrayBitmap IsNot Nothing Then
            Gcode_Height = w * GrayBitmap.Height / GrayBitmap.Width
        End If
    End Sub

    Friend Sub Gcode_SetHeight(ByVal h As Single)
        Gcode_Height = h
        If Gcode_UseImageAspect AndAlso GrayBitmap IsNot Nothing Then
            Gcode_Width = h * GrayBitmap.Width / GrayBitmap.Height
        End If
    End Sub

    Friend Function Gcode_GetOutWidth() As Single
        If Gcode_UseImageSize AndAlso GrayBitmap IsNot Nothing Then
            Return 25.4F * GrayBitmap.Width / GrayBitmap.HorizontalResolution
        Else
            Return Gcode_Width
        End If
    End Function
    Friend Function Gcode_GetOutHeight() As Single
        If Gcode_UseImageSize AndAlso GrayBitmap IsNot Nothing Then
            Return 25.4F * GrayBitmap.Height / GrayBitmap.VerticalResolution
        Else
            Return Gcode_Height
        End If
    End Function

    Friend Sub Gcode_CenterImage(ByVal Centered As Boolean, ByVal PaperWidth As Single, ByVal PaperHeight As Single)
        If Centered Then
            Gcode_ZeroX = Max((PaperWidth - Gcode_Width) / 2, 0)
            Gcode_ZeroY = Max((PaperHeight - Gcode_Height) / 2, 0)
        Else
            Gcode_ZeroX = 0
            Gcode_ZeroY = 0
        End If
    End Sub


    Friend Sub Gcode_FromVectorList(ByVal FileName As String)
        '
        'Dim sw As Stopwatch = New Stopwatch
        'sw.Start()
        '
        If VecList Is Nothing Then Return
        If GrayBitmap Is Nothing Then Return
        Dim vl As VectorList = VecList
        Dim w As Int32 = GrayBitmap.Width
        Dim h As Int32 = GrayBitmap.Height
        Dim kx As Single
        Dim ky As Single
        If Gcode_UseImageSize Then
            kx = 25.4F / GrayBitmap.HorizontalResolution * GrayBitmap.Width
            ky = 25.4F / GrayBitmap.VerticalResolution * GrayBitmap.Height
        Else
            kx = Gcode_Width
            ky = Gcode_Height
        End If
        ' --------------------------------------------------------
        SetFileName(FileName)
        SetMaxValues(Gcode_Rapids, Gcode_Acceleration)
        SetStock(w * kx, h * ky, -Gcode_Zdown + 1)
        SetRapidFeed(Gcode_Rapids)
        SetTool(Gcode_ToolSize, 100)
        Start()
        ' --------------------------------------------------------
        SetFeed(Gcode_Feed)
        SetSpeed(Gcode_Speed)
        ' --------------------------------------------------------
        Dim x As Single
        Dim y As Single
        Dim FirstPoint As Boolean
        For Each vec As Vector In vl.vectors
            FirstPoint = True
            For Each p As PointF In vec.points
                x = Gcode_ZeroX + p.X * kx
                y = Gcode_ZeroY + (1 - p.Y) * ky
                If FirstPoint Then
                    MoveTo(Gcode_Zup)
                    RapidTo(x, y)
                    MoveTo(Gcode_Zdown)
                    FirstPoint = False
                Else
                    MoveTo(x, y)
                End If
            Next
        Next
        ' --------------------------------------------------------
        Finish()
        '
        'Form1.Text = sw.Elapsed.TotalMilliseconds.ToString
    End Sub

    Friend Sub Gcode_WriteFile()
        ' ----------------------------------------------------------- delete old file and write new file
        File_KillFile(FileName)
        Dim file As IO.StreamWriter = My.Computer.FileSystem.OpenTextFileWriter(FileName, _
                                                                                False, _
                                                                                System.Text.Encoding.ASCII)
        For Each s As String In Gcode
            file.WriteLine(s)
        Next
        file.Close()
        '
        ' ----------------------------------------------------------- start Gcode visualizer application
        Try
            Process.Start(FileName)
        Catch ex As Exception
            MessageBox.Show("Error: " + ex.Message + vbCrLf + vbCrLf + "Can not start the file : """ + FileName + """")
        End Try
    End Sub

    Friend Function Gcode_GetTotalTime() As String
        Dim t As Double = TotTime
        Dim m As Integer = CInt(Floor(t / 60.0))
        t -= m * 60
        Dim sec As Integer = CInt(Floor(t))
        '
        If m > 99 Then
            Return m.ToString + " min "
        Else
            Return m.ToString + " min " + sec.ToString + " sec"
        End If
    End Function


    ' =========================================================================
    '   GCODE GENERATION
    ' =========================================================================
    Private CoordFormat As String = "0.000"

    Private FileName As String

    Private old_X As Double
    Private old_Y As Double
    Private old_Z As Double

    Private TotTime As Double
    Private TotRapid As Double
    Private TotMove As Double

    Private MaxFeedX As Double
    Private MaxFeedY As Double
    Private MaxFeedZ As Double

    Private AccelX As Double
    Private AccelY As Double
    Private AccelZ As Double

    Private WorkFeed As Double
    Private WorkSpeed As Double
    Private RapidFeed As Double

    Private ToolDiam As Double
    Private ToolAngle As Double

    Private stock_dx As Double
    Private stock_dy As Double
    Private stock_dz As Double

    Private stock_ox As Double
    Private stock_oy As Double
    Private stock_oz As Double

    Private Gcode As List(Of String)


    ' =======================================================================================
    '   Private functions
    ' =======================================================================================
    Private Sub Start()
        Gcode = New List(Of String)

        TotTime = 0
        TotMove = 0
        TotRapid = 0
        Gcode.Clear()
        Gcode.Add("")
        Gcode.Add("(MCUSTOCK X" & FormatNum(stock_dx) & _
                        " Y" & FormatNum(stock_dy) & _
                        " Z" & FormatNum(stock_dz) & _
                        " OTL" & _
                        " OX" & FormatNum(stock_ox) & _
                        " OY" & FormatNum(stock_oy) & _
                        " OZ" & FormatNum(stock_oz) & ")")
        Gcode.Add("")
        Gcode.Add("G21")
        Gcode.Add("G90")
        Gcode.Add("G40")
        Gcode.Add("M03")
        Gcode.Add("")
        Gcode.Add("(MCUTOOL T13 D" & FormatNum(ToolDiam) & " F3 L30 A" & FormatNum(ToolAngle) & ")")
        Gcode.Add("M06 T13")
        Gcode.Add("")
    End Sub

    Private Sub Finish()
        Gcode.Add("")
        Gcode.Add("(==============================)")
        Gcode.Add("( END OF PROGRAM               )")
        Gcode.Add("M05")
        MoveTo(2)
        RapidTo(0, 0, 5)
        Gcode.Add("M30")
        Gcode.Add("M02")
        Gcode.Add("(==============================)")
        Gcode.Add("")
        Gcode.Add("(======= STATISTICS =======)")
        Gcode.Add("( Total Rapid " & TotRapid.ToString("0") & " mm")
        Gcode.Add("( Total Move " & TotMove.ToString("0") & " mm")
        Gcode.Add("")
        Gcode.Add("( Total estimated time:")
        '
        Dim t As Double = TotTime
        Dim h As Integer = CInt(Floor(t / 3600))
        t -= h * 3600
        Dim m As Integer = CInt(Floor(t / 60.0))
        t -= m * 60
        Dim sec As Integer = CInt(Floor(t))
        '
        If h > 0 Then Gcode.Add("( " & h.ToString & " hours")
        If m > 0 Then Gcode.Add("( " & m.ToString & " min.")
        Gcode.Add("( " & sec.ToString & " sec.")
        Gcode.Add("")
    End Sub

    Private Sub SetFileName(ByVal _FileName As String)
        FileName = IO.Path.GetDirectoryName(_FileName) & "\" & IO.Path.GetFileNameWithoutExtension(_FileName) & ".gc"
    End Sub

    Private Sub SetMaxValues(ByVal MaxFeed As Double, ByVal Acceleration As Double)
        MaxFeedX = MaxFeed
        MaxFeedY = MaxFeed
        MaxFeedZ = MaxFeed
        AccelX = Acceleration
        AccelY = Acceleration
        AccelZ = Acceleration
    End Sub

    Private Sub SetStock(ByVal x As Double, ByVal y As Double, ByVal z As Double)
        stock_dx = x
        stock_dy = y
        stock_dz = z
    End Sub

    Private Sub SetRapidFeed(ByVal _RapidFeed As Double)
        RapidFeed = _RapidFeed
    End Sub

    Private Sub SetFeed(ByVal _WorkFeed As Double)
        WorkFeed = _WorkFeed
        Gcode.Add("F " & WorkFeed.ToString)
    End Sub

    Private Sub SetSpeed(ByVal _WorkSpeed As Double)
        WorkSpeed = _WorkSpeed
        Gcode.Add("S " & WorkSpeed.ToString)
    End Sub

    Private Sub SetTool(ByVal _ToolDiam As Double, ByVal _ToolAngle As Double)
        ToolDiam = _ToolDiam
        ToolAngle = _ToolAngle
    End Sub

    Private Sub RapidTo(ByVal x As Double, ByVal y As Double, ByVal z As Double)
        If x <> old_X Or y <> old_Y Or z <> old_Z Then
            '
            Dim delta As Double
            '
            If Abs(z - old_Z) > Abs(x - old_X) And Abs(z - old_Z) > Abs(y - old_Y) Then
                delta = Abs(z - old_Z)
                TotTime += CalcTime(delta, MaxFeedZ, AccelZ)
            Else
                If Abs(x - old_X) > Abs(y - old_Y) Then
                    delta = Abs(x - old_X)
                    TotTime += CalcTime(delta, MaxFeedX, AccelX)
                Else
                    delta = Abs(y - old_Y)
                    TotTime += CalcTime(delta, MaxFeedY, AccelY)
                End If
            End If
            '
            'TotRapid += delta
            TotRapid += Dist(x, old_X, y, old_Y, z, old_Z)
            '
            old_X = x
            old_Y = y
            old_Z = z
            '
            Gcode.Add("G00 X" & FormatNum(x) & " Y" & FormatNum(y) & " Z" & FormatNum(z))
        End If
    End Sub

    Private Sub RapidTo(ByVal x As Double, ByVal y As Double)
        If x <> old_X Or y <> old_Y Then
            '
            Dim delta As Double
            '
            If Abs(x - old_X) > Abs(y - old_Y) Then
                delta = Abs(x - old_X)
                TotTime += CalcTime(delta, MaxFeedX, AccelX)
            Else
                delta = Abs(y - old_Y)
                TotTime += CalcTime(delta, MaxFeedY, AccelY)
            End If
            '
            'TotRapid += delta
            TotRapid += Dist(x, old_X, y, old_Y)
            '
            old_X = x
            old_Y = y
            '
            Gcode.Add("G00 X" & FormatNum(x) & " Y" & FormatNum(y))
        End If
    End Sub

    Private Sub RapidTo(ByVal z As Double)
        '
        Dim delta As Double
        '
        If z <> old_Z Then
            delta = Abs(z - old_Z)
            TotTime += CalcTime(delta, MaxFeedZ, AccelZ)
            TotRapid += delta
            '
            old_Z = z
            '
            Gcode.Add("G00 Z" & FormatNum(z))
        End If
    End Sub

    Private Sub MoveTo(ByVal x As Double, ByVal y As Double, ByVal z As Double)
        If x <> old_X Or y <> old_Y Or z <> old_Z Then
            '
            Dim delta As Double
            '
            If Abs(z - old_Z) > Abs(x - old_X) And Abs(z - old_Z) > Abs(y - old_Y) Then
                delta = Abs(z - old_Z)
                TotTime += CalcTime(delta, MaxFeedZ, AccelZ)
            Else
                If Abs(x - old_X) > Abs(y - old_Y) Then
                    delta = Abs(x - old_X)
                    TotTime += CalcTime(delta, WorkFeed, AccelX)
                Else
                    delta = Abs(y - old_Y)
                    TotTime += CalcTime(delta, WorkFeed, AccelY)
                End If
            End If
            '
            'TotMove += delta
            TotMove += Dist(x, old_X, y, old_Y, z, old_Z)
            '
            old_X = x
            old_Y = y
            old_Z = z
            '
            Gcode.Add("G01 X" & FormatNum(x) & " Y" & FormatNum(y) & " Z" & FormatNum(z))
        End If
    End Sub

    Private Sub MoveTo(ByVal x As Double, ByVal y As Double)
        If x <> old_X Or y <> old_Y Then
            '
            Dim delta As Double
            '
            If Abs(x - old_X) > Abs(y - old_Y) Then
                delta = Abs(x - old_X)
                TotTime += CalcTime(delta, WorkFeed, AccelX)
            Else
                delta = Abs(y - old_Y)
                TotTime += CalcTime(delta, WorkFeed, AccelY)
            End If
            '
            'TotMove += delta
            TotMove += Dist(x, old_X, y, old_Y)
            '
            old_X = x
            old_Y = y
            '
            Gcode.Add("G01 X" & FormatNum(x) & " Y" & FormatNum(y))
        End If
    End Sub

    Private Sub MoveTo(ByVal z As Double)
        '
        Dim delta As Double
        '
        If z <> old_Z Then
            delta = Abs(z - old_Z)
            TotTime += CalcTime(delta, WorkFeed, AccelZ)
            TotRapid += delta
            '
            old_Z = z
            '
            Gcode.Add("G01 Z" & FormatNum(z))
        End If
    End Sub


    ' =======================================================================================
    '   helper functions
    ' =======================================================================================
    Private Function Dist(ByVal x1 As Double, ByVal x2 As Double, ByVal y1 As Double, ByVal y2 As Double, ByVal z1 As Double, ByVal z2 As Double) As Double
        Return Math.Sqrt((x2 - x1) ^ 2 + (y2 - y1) ^ 2 + (z2 - z1) ^ 2)
    End Function

    Private Function Dist(ByVal x1 As Double, ByVal x2 As Double, ByVal y1 As Double, ByVal y2 As Double) As Double
        Return Math.Sqrt((x2 - x1) ^ 2 + (y2 - y1) ^ 2)
    End Function


    Private Function CalcTime(ByVal TotalSpace As Double, _
                              ByVal SpeedMax As Double, _
                              ByVal Accel As Double) As Double

        CalcTime = CalcMotionTime(TotalSpace, SpeedMax, Accel)
        ' --------------------------------------------------------- experimental correction
        CalcTime *= 1.0
    End Function

    ' -------------------------------------------------------------------
    '  Compute the motion time for linear acceleration and deceleration 
    ' -------------------------------------------------------------------
    ' TotalSpace (mm)          (3500)
    ' SpeedMax   (mm/min)      (3500)
    ' Accel      (mm/s/s)      (1050)
    ' -------------------------------------------------------------------
    Private Function CalcMotionTime(ByVal TotalSpace As Double, _
                                     ByVal SpeedMax As Double, _
                                     ByVal Accel As Double) As Double
        '
        If TotalSpace <= 0 Or SpeedMax <= 0 Or Accel <= 0 Then Return 0
        Dim SpeedMaxMM As Double = SpeedMax / 60
        Dim AccelTime As Double = SpeedMaxMM / Accel
        Dim AccelSpace As Double = Accel * AccelTime * AccelTime
        Dim ConstVelSpace As Double = TotalSpace - AccelSpace * 2
        If ConstVelSpace > 0 Then
            ' ------------------------------ trapezoidal
            Dim ConstVelTime As Double = ConstVelSpace / SpeedMaxMM
            Return AccelTime * 2 + ConstVelTime
        Else
            ' ------------------------------ triangle
            Return 2 * Sqrt(TotalSpace / Accel)
        End If
    End Function


    Private Sub File_KillFile(ByRef FileName As String)
        On Error Resume Next
        If Not My.Computer.FileSystem.FileExists(FileName) Then Exit Sub
        SetAttr(FileName, FileAttribute.Normal)
        Kill(FileName)
    End Sub

    Private Function FormatNum(ByVal n As Double) As String
        Return n.ToString(CoordFormat).Replace(",", ".")
    End Function

End Module
