﻿
Imports System.Drawing.Imaging
Imports System.Drawing.Drawing2D


Module Module_ImageFilters

    ' ========================================================
    '  RESIZE AND CORRECT DPI TO RESTORE THE SAME PRINT SIZE
    ' ========================================================
    Friend Function ImageResize(ByRef bmp1 As Image, ByVal w As Integer, ByVal h As Integer, ByVal quality As Int32) As Image
        If bmp1 Is Nothing Then Return Nothing
        Dim bmp2 As Bitmap = New Bitmap(w, h)
        bmp2.SetResolution(bmp1.HorizontalResolution * w / bmp1.Width, bmp1.VerticalResolution * h / bmp1.Height)
        Dim g As Graphics = Graphics.FromImage(bmp2)
        SetQuality(g, quality)
        g.DrawImage(bmp1, 0, 0, w, h)
        Return bmp2
    End Function

    Friend Function BmpResize(ByRef bmp1 As Bitmap, ByVal w As Integer, ByVal h As Integer, ByVal quality As Int32) As Bitmap
        If bmp1 Is Nothing Then Return Nothing
        Dim bmp2 As Bitmap = New Bitmap(w, h)
        bmp2.SetResolution(bmp1.HorizontalResolution * w / bmp1.Width, bmp1.VerticalResolution * h / bmp1.Height)
        Dim g As Graphics = Graphics.FromImage(bmp2)
        SetQuality(g, quality)
        g.DrawImage(bmp1, 0, 0, w, h)
        Return bmp2
    End Function

    'Friend Sub ImageResizeInPlace(ByRef bmp1 As Image, ByVal w As Integer, ByVal h As Integer, ByVal quality As Int32)
    '    If bmp1 Is Nothing Then Exit Sub
    '    Dim bmp2 As Bitmap = New Bitmap(w, h)
    '    bmp2.SetResolution(bmp1.HorizontalResolution * w / bmp1.Width, bmp1.VerticalResolution * h / bmp1.Height)
    '    Dim g As Graphics = Graphics.FromImage(bmp2)
    '    SetQuality(g, quality)
    '    g.DrawImage(bmp1, 0, 0, w, h)
    '    bmp1 = bmp2
    'End Sub

    Friend Sub ResizeBitmap(ByRef bmp As Bitmap, ByVal Resolution As Int32)
        ' -------------------------------------------------- create a resized bitmap with the same DPI 
        Dim w As Int32 = bmp.Width
        Dim h As Int32 = bmp.Height
        Dim k As Single = CSng(Resolution) / w
        bmp = BmpResize(bmp, CInt(w * k), CInt(h * k), 9)
    End Sub

    Friend Sub BlurBitmap(ByRef bmp As Bitmap, ByVal BlurPixels As Single)
        If BlurPixels <= 0 Then Return
        ' ---------------------------------------------
        Dim w As Int32 = bmp.Width
        Dim h As Int32 = bmp.Height
        Dim w2 As Int32 = CInt(w / (BlurPixels + 1))
        Dim h2 As Int32 = CInt(h / (BlurPixels + 1))
        If w2 < 2 Then w2 = 2
        If h2 < 2 Then h2 = 2
        ' --------------------------------------------- low-resolution
        Dim bmp2 As New Bitmap(w2, h2)
        Dim g As Graphics = Graphics.FromImage(bmp2)
        g.InterpolationMode = InterpolationMode.HighQualityBilinear
        g.DrawImage(bmp, 0, 0, w2, h2)
        ' --------------------------------------------- hi resolution blurred
        g = Graphics.FromImage(bmp)
        g.Clear(Color.White)
        g.InterpolationMode = InterpolationMode.HighQualityBilinear
        g.DrawImage(bmp2, 0, 0, w, h)
    End Sub

    Friend Sub ProcessImage_Gray(ByRef bmp As Bitmap)
        Dim imgAttr As ImageAttributes = New ImageAttributes()
        imgAttr.SetColorMatrix(New ColorMatrix(New Single()() _
                               {New Single() {0.299, 0.299, 0.299, 0, 0}, _
                                New Single() {0.587, 0.587, 0.587, 0, 0}, _
                                New Single() {0.114, 0.114, 0.114, 0, 0}, _
                                New Single() {0, 0, 0, 1, 0}, _
                                New Single() {0, 0, 0, 0, 1}}))
        Dim g As Graphics = Graphics.FromImage(bmp)
        Dim w As Integer = bmp.Width
        Dim h As Integer = bmp.Height
        Dim rec As Rectangle = New Rectangle(0, 0, w, h)
        g.DrawImage(bmp, rec, 0, 0, w, h, GraphicsUnit.Pixel, imgAttr)
    End Sub

    Friend Sub ProcessImage_Contrast(ByRef img As Image)
        Dim imgAttr As ImageAttributes = New ImageAttributes()
        imgAttr.SetColorMatrix(New ColorMatrix(New Single()() _
                               {New Single() {2, 0, 0, 0, 0}, _
                                New Single() {0, 2, 0, 0, 0}, _
                                New Single() {0, 0, 2, 0, 0}, _
                                New Single() {0, 0, 0, 1, 0}, _
                                New Single() {0, 0, 0, 0, 1}}))
        Dim g As Graphics = Graphics.FromImage(img)
        Dim w As Integer = img.Width
        Dim h As Integer = img.Height
        Dim rec As Rectangle = New Rectangle(0, 0, w, h)
        g.DrawImage(img, rec, 0, 0, w, h, GraphicsUnit.Pixel, imgAttr)
    End Sub


    ' ========================================================
    '  BMP Change Format to 24bit
    ' ========================================================
    Friend Function BmpChangeTo24Bit(ByVal bmp As Bitmap) As Bitmap
        If bmp Is Nothing Then Return Nothing
        Dim w As Int32 = bmp.Width
        Dim h As Int32 = bmp.Height
        Dim b2 As Bitmap = New Bitmap(w, h, Imaging.PixelFormat.Format24bppRgb)
        b2.SetResolution(bmp.HorizontalResolution, bmp.VerticalResolution)
        Dim g As Graphics = Graphics.FromImage(b2)
        'g.InterpolationMode = InterpolationMode.NearestNeighbor
        g.DrawImage(bmp, 0, 0, w, h)
        Return b2
    End Function

    ' ================================================================================
    '  QUALITY
    ' ================================================================================
    Private Sub SetQuality(ByVal g As Graphics, ByVal nQuality As Int32)
        Select Case nQuality
            Case 1
                g.CompositingQuality = CompositingQuality.HighSpeed
                g.SmoothingMode = SmoothingMode.HighSpeed
                g.InterpolationMode = InterpolationMode.NearestNeighbor
            Case 2
                g.CompositingQuality = CompositingQuality.HighSpeed
                g.SmoothingMode = SmoothingMode.HighSpeed
                g.InterpolationMode = InterpolationMode.Low
            Case 3
                g.CompositingQuality = CompositingQuality.HighSpeed
                g.SmoothingMode = SmoothingMode.HighSpeed
                g.InterpolationMode = InterpolationMode.Bilinear
            Case 4
                g.CompositingQuality = CompositingQuality.HighSpeed
                g.SmoothingMode = SmoothingMode.HighSpeed
                g.InterpolationMode = InterpolationMode.HighQualityBilinear
            Case 5
                g.CompositingQuality = CompositingQuality.HighQuality
                g.SmoothingMode = SmoothingMode.HighQuality
                g.InterpolationMode = InterpolationMode.NearestNeighbor
            Case 6
                g.CompositingQuality = CompositingQuality.HighSpeed
                g.SmoothingMode = SmoothingMode.HighSpeed
                g.InterpolationMode = InterpolationMode.Bicubic
            Case 7
                g.CompositingQuality = CompositingQuality.HighQuality
                g.SmoothingMode = SmoothingMode.HighQuality
                g.InterpolationMode = InterpolationMode.Bicubic
            Case 8
                g.CompositingQuality = CompositingQuality.HighQuality
                g.SmoothingMode = SmoothingMode.HighQuality
                g.InterpolationMode = InterpolationMode.HighQualityBilinear
            Case 9
                g.CompositingQuality = CompositingQuality.HighQuality
                g.SmoothingMode = SmoothingMode.HighQuality
                g.InterpolationMode = InterpolationMode.HighQualityBicubic
                ' --------------------------------------------------------------- ( zero or > 9 ) 
            Case Else
                g.CompositingQuality = CompositingQuality.HighSpeed
                g.SmoothingMode = SmoothingMode.HighSpeed
                g.InterpolationMode = InterpolationMode.Low
        End Select
    End Sub


End Module