﻿
Imports System.Collections.Generic

Module Module_Vectorize

    Friend Class Vector
        Friend points As List(Of PointF)
        Friend Sub New()
            points = New List(Of PointF)
        End Sub
    End Class

    Friend Class VectorList
        Friend vectors As List(Of Vector)
        Friend Sub New()
            vectors = New List(Of Vector)
        End Sub
    End Class

    Friend VecList As VectorList
    Private PixArray(-1, -1) As Byte
    Private sizeX As Integer
    Private sizeY As Integer

    Friend RowsNumber As Int32 = 100
    Friend PointsNumber As Int32 = 500
    Friend ZigZagPercentual As Int32 = 100
    Friend WhiteLevel As Int32
    Friend MidLevel As Int32
    Friend BlackLevel As Int32

    ' ========================================================================================================
    '  Vectors to Image
    ' ========================================================================================================
    Friend Function VectorsToImage(ByVal w As Int32, ByVal h As Int32) As Bitmap
        '
        If w < 1 Or h < 1 Then Return Nothing
        Dim bmp As Bitmap = New Bitmap(w, h)
        Dim g As Graphics = Graphics.FromImage(bmp)
        g.SmoothingMode = Drawing2D.SmoothingMode.HighSpeed
        g.InterpolationMode = Drawing2D.InterpolationMode.HighQualityBicubic
        g.Clear(Color.White)
        If VecList Is Nothing Then Return bmp
        '
        Dim oldx As Single
        Dim oldy As Single
        Dim x, y As Single
        '
        For Each v As Vector In VecList.vectors
            oldx = -999
            For Each p As PointF In v.points
                x = p.X * w
                y = p.Y * h
                If oldx >= 0 Then
                    g.DrawLine(Pens.Black, oldx, oldy, x, y)
                End If
                oldx = x
                oldy = y
            Next
        Next
        Return bmp
    End Function


    ' ========================================================================================================
    '  CreateVectors H
    ' ========================================================================================================
    Friend Sub CreateVectors_H()
        If GrayBitmap Is Nothing Then Return
        ' ------------------------------------------------------- Image dimensions and aspect
        sizeX = GrayBitmap.Width
        sizeY = GrayBitmap.Height
        Dim Aspect As Single = CSng(sizeY / sizeX)
        ' ------------------------------------------------------- Bitmap to byte array
        GrayBitmap_to_ByteArray(GrayBitmap, PixArray)
        ' ------------------------------------------------------- Init vars
        Dim DeltaYmax As Single = ZigZagPercentual / 100.0F / RowsNumber
        DeltaYmax /= (MidLevel - BlackLevel)
        Dim DeltaY As Single
        Dim Blackness As Single
        Dim Counter As Single
        Dim v As Single
        Dim vec As Vector
        Dim ImgX As Int32
        Dim ImgY As Int32
        Dim PosX As Single
        Dim PosY As Single
        Dim PointStart As Int32
        Dim PointEnd As Int32
        Dim PointStep As Int32
        ' ------------------------------------------------------- for each row
        For y As Int32 = 0 To RowsNumber - 1
            ImgY = CInt(Math.Floor(y * sizeY / RowsNumber))
            vec = New Vector
            ' --------------------------------------------------- bidirectional
            If y Mod 2 = 0 Then
                PointStart = 0
                PointEnd = PointsNumber - 1
                PointStep = 1
            Else
                PointStart = PointsNumber - 1
                PointEnd = 0
                PointStep = -1
            End If
            ' --------------------------------------------------- for each point
            For x As Int32 = PointStart To PointEnd Step PointStep
                ImgX = CInt(Math.Floor(x * sizeX / PointsNumber))
                ' ----------------------------------------------- get grey level
                v = PixArray(ImgX, ImgY)
                ' ----------------------------------------------- 
                PosX = CSng(x / PointsNumber)
                PosY = CSng(y / RowsNumber)
                ' ----------------------------------------------- mid to black
                If v < MidLevel Then
                    If v < BlackLevel Then v = BlackLevel
                    DeltaY = DeltaYmax * (MidLevel - v)
                    If (x Mod 2) = 0 Then DeltaY = -DeltaY
                    vec.points.Add(New PointF(PosX, PosY + DeltaY))
                    ' ------------------------------------------- white to mid
                ElseIf v < WhiteLevel And WhiteLevel <> MidLevel Then
                    vec.points.Add(New PointF(PosX, PosY))
                    Blackness = (WhiteLevel - v) / (WhiteLevel - MidLevel)
                    If Counter < Blackness Then
                        Counter += 0.2F
                    Else
                        Counter = 0
                        If vec.points.Count > 0 Then
                            VecList.vectors.Add(vec)
                            vec = New Vector
                        End If
                    End If
                    ' ------------------------------------------- more than white
                Else
                    If vec.points.Count > 0 Then
                        VecList.vectors.Add(vec)
                        vec = New Vector
                    End If
                End If
            Next
            If vec.points.Count > 0 Then
                VecList.vectors.Add(vec)
                vec = New Vector
            End If
        Next
    End Sub


    ' ========================================================================================================
    '  CreateVectors V
    ' ========================================================================================================
    Friend Sub CreateVectors_V()
        If GrayBitmap Is Nothing Then Return
        ' ------------------------------------------------------- Image dimensions and aspect
        sizeX = GrayBitmap.Width
        sizeY = GrayBitmap.Height
        Dim Aspect As Single = CSng(sizeY / sizeX)
        ' ------------------------------------------------------- correct params using the image aspect
        Dim PointN As Int32 = CInt(PointsNumber / Aspect)
        Dim RowsN As Int32 = CInt(RowsNumber / Aspect)
        ' ------------------------------------------------------- Bitmap to byte array
        GrayBitmap_to_ByteArray(GrayBitmap, PixArray)
        ' ------------------------------------------------------- Init vars
        Dim DeltaXmax As Single = ZigZagPercentual / 100.0F / RowsN
        DeltaXmax /= (MidLevel - BlackLevel)
        Dim DeltaX As Single
        Dim Blackness As Single
        Dim Counter As Single
        Dim v As Single
        Dim vec As Vector
        Dim ImgX As Int32
        Dim ImgY As Int32
        Dim PosX As Single
        Dim PosY As Single
        Dim PointStart As Int32
        Dim PointEnd As Int32
        Dim PointStep As Int32
        ' ------------------------------------------------------- for each vertical row
        For xx As Int32 = 0 To RowsN - 1
            ImgX = CInt(Math.Floor(xx * sizeX / RowsN))
            vec = New Vector
            ' --------------------------------------------------- bidirectional
            If xx Mod 2 = 0 Then
                PointStart = 0
                PointEnd = PointN - 1
                PointStep = 1
            Else
                PointStart = PointN - 1
                PointEnd = 0
                PointStep = -1
            End If
            ' --------------------------------------------------- for each point
            For yy As Int32 = PointStart To PointEnd Step PointStep
                ImgY = CInt(Math.Floor(yy * sizeY / PointN))
                ' ----------------------------------------------- get grey level
                v = PixArray(ImgX, ImgY)
                ' ----------------------------------------------- 
                PosY = CSng(yy / PointN)
                PosX = CSng(xx / RowsN)
                ' ----------------------------------------------- mid to black
                If v < MidLevel Then
                    If v < BlackLevel Then v = BlackLevel
                    DeltaX = DeltaXmax * (MidLevel - v)
                    If (yy Mod 2) = 0 Then DeltaX = -DeltaX
                    vec.points.Add(New PointF(PosX + DeltaX, PosY))
                    ' ------------------------------------------- white to mid
                ElseIf v < WhiteLevel And WhiteLevel <> MidLevel Then
                    vec.points.Add(New PointF(PosX, PosY))
                    Blackness = (WhiteLevel - v) / (WhiteLevel - MidLevel)
                    If Counter < Blackness Then
                        Counter += 0.2F
                    Else
                        Counter = 0
                        If vec.points.Count > 0 Then
                            VecList.vectors.Add(vec)
                            vec = New Vector
                        End If
                    End If
                    ' ------------------------------------------- more than white
                Else
                    If vec.points.Count > 0 Then
                        VecList.vectors.Add(vec)
                        vec = New Vector
                    End If
                End If
            Next
            If vec.points.Count > 0 Then
                VecList.vectors.Add(vec)
                vec = New Vector
            End If
        Next
    End Sub

 
End Module
